@@ -22,11 +22,44 @@
 from nuitka import Options, Variables
 from nuitka.utils import Utils
 
-from . import CodeTemplates
 from .ConstantCodes import getConstantCode
 from .Emission import SourceCodeCollector
-from .ErrorCodes import getErrorFormatExitBoolCode, getErrorFormatExitCode
+from .ErrorCodes import (
+    getAssertionCode,
+    getErrorFormatExitBoolCode,
+    getErrorFormatExitCode
+)
 from .Indentation import indented
+from .templates.CodeTemplatesVariables import (
+    template_check_local,
+    template_check_shared,
+    template_del_global_unclear,
+    template_del_local_intolerant,
+    template_del_local_known,
+    template_del_local_tolerant,
+    template_del_shared_intolerant,
+    template_del_shared_known,
+    template_del_shared_tolerant,
+    template_read_local,
+    template_read_maybe_local_unclear,
+    template_read_mvar_unclear,
+    template_read_shared_known,
+    template_read_shared_unclear,
+    template_release_clear,
+    template_release_unclear,
+    template_write_local_clear_ref0,
+    template_write_local_clear_ref1,
+    template_write_local_empty_ref0,
+    template_write_local_empty_ref1,
+    template_write_local_inplace,
+    template_write_local_unclear_ref0,
+    template_write_local_unclear_ref1,
+    template_write_shared_clear_ref0,
+    template_write_shared_clear_ref1,
+    template_write_shared_inplace,
+    template_write_shared_unclear_ref0,
+    template_write_shared_unclear_ref1
+)
 
 
 def generateVariableDelCode(statement, emit, context):
@@ -95,7 +128,7 @@ def _getLocalVariableCode(context, variable):
 
     if owner is user:
         result = getVariableCodeName(
-            in_context = owner is not user,
+            in_context = False,
             variable   = variable
         )
 
@@ -125,7 +158,7 @@ def _getLocalVariableCode(context, variable):
                 variable   = variable
             )
 
-            return result, False
+            return result, variable.isSharedTechnically()
     else:
         closure_index = user.getClosureVariables().index(variable)
 
@@ -217,39 +250,39 @@ def getVariableAssignmentCode(context, emit, variable, tmp_name, needs_release,
     elif variable.isLocalVariable():
         if in_place:
             # Releasing is not an issue here, local variable reference never
-            # gave a reference, and the inplace code deals with possible
+            # gave a reference, and the in-place code deals with possible
             # replacement/release.
             if variable.isSharedTechnically():
-                template = CodeTemplates.template_write_shared_inplace
+                template = template_write_shared_inplace
             else:
-                template = CodeTemplates.template_write_local_inplace
+                template = template_write_local_inplace
 
         elif variable.isSharedTechnically():
             if ref_count:
                 if needs_release is False:
-                    template = CodeTemplates.template_write_shared_clear_ref0
+                    template = template_write_shared_clear_ref0
                 else:
-                    template = CodeTemplates.template_write_shared_unclear_ref0
+                    template = template_write_shared_unclear_ref0
             else:
                 if needs_release is False:
-                    template = CodeTemplates.template_write_shared_clear_ref1
+                    template = template_write_shared_clear_ref1
                 else:
-                    template = CodeTemplates.template_write_shared_unclear_ref1
+                    template = template_write_shared_unclear_ref1
         else:
             if ref_count:
                 if needs_release is False:
-                    template = CodeTemplates.template_write_local_empty_ref0
+                    template = template_write_local_empty_ref0
                 elif needs_release is True:
-                    template = CodeTemplates.template_write_local_clear_ref0
+                    template = template_write_local_clear_ref0
                 else:
-                    template = CodeTemplates.template_write_local_unclear_ref0
+                    template = template_write_local_unclear_ref0
             else:
                 if needs_release is False:
-                    template = CodeTemplates.template_write_local_empty_ref1
+                    template = template_write_local_empty_ref1
                 elif needs_release is True:
-                    template = CodeTemplates.template_write_local_clear_ref1
+                    template = template_write_local_clear_ref1
                 else:
-                    template = CodeTemplates.template_write_local_unclear_ref1
+                    template = template_write_local_unclear_ref1
 
         emit(
             template % {
@@ -263,24 +296,24 @@ def getVariableAssignmentCode(context, emit, variable, tmp_name, needs_release,
     elif variable.isTempVariable():
         if variable.isSharedTechnically():
             if ref_count:
-                template = CodeTemplates.template_write_shared_unclear_ref0
+                template = template_write_shared_unclear_ref0
             else:
-                template = CodeTemplates.template_write_shared_unclear_ref1
+                template = template_write_shared_unclear_ref1
         else:
             if ref_count:
                 if needs_release is False:
-                    template = CodeTemplates.template_write_local_empty_ref0
+                    template = template_write_local_empty_ref0
                 elif needs_release is True:
-                    template = CodeTemplates.template_write_local_clear_ref0
+                    template = template_write_local_clear_ref0
                 else:
-                    template = CodeTemplates.template_write_local_unclear_ref0
+                    template = template_write_local_unclear_ref0
             else:
                 if needs_release is False:
-                    template = CodeTemplates.template_write_local_empty_ref1
+                    template = template_write_local_empty_ref1
                 elif needs_release is True:
-                    template = CodeTemplates.template_write_local_clear_ref1
+                    template = template_write_local_clear_ref1
                 else:
-                    template = CodeTemplates.template_write_local_unclear_ref1
+                    template = template_write_local_unclear_ref1
 
         emit(
             template % {
@@ -302,7 +335,7 @@ def getVariableAccessCode(to_name, variable, needs_check, emit, context):
 
     if variable.isModuleVariable():
         emit(
-            CodeTemplates.template_read_mvar_unclear % {
+            template_read_mvar_unclear % {
                 "module_identifier" : context.getModuleCodeName(),
                 "tmp_name"          : to_name,
                 "var_name"          : getConstantCode(
@@ -313,7 +346,9 @@ def getVariableAccessCode(to_name, variable, needs_check, emit, context):
         )
 
         if needs_check:
-            if Utils.python_version < 340 and not context.isPythonModule():
+            if Utils.python_version < 340 and \
+               not context.isPythonModule() and \
+               not context.getOwner().isClassDictCreation():
                 error_message = "global name '%s' is not defined"
             else:
                 error_message = "name '%s' is not defined"
@@ -343,7 +378,7 @@ def getVariableAccessCode(to_name, variable, needs_check, emit, context):
         )
 
         emit(
-            CodeTemplates.template_read_maybe_local_unclear % {
+            template_read_maybe_local_unclear % {
                 "locals_dict" : "locals_dict",
                 "fallback"    : indented(fallback_emit.codes),
                 "tmp_name"    : to_name,
@@ -358,11 +393,11 @@ def getVariableAccessCode(to_name, variable, needs_check, emit, context):
     elif variable.isLocalVariable():
         if variable.isSharedTechnically():
             if not needs_check:
-                template = CodeTemplates.template_read_shared_unclear
+                template = template_read_shared_unclear
             else:
-                template = CodeTemplates.template_read_shared_known
+                template = template_read_shared_known
         else:
-            template = CodeTemplates.template_read_local
+            template = template_read_local
 
         emit(
             template % {
@@ -404,7 +439,7 @@ local variable '%s' referenced before assignment""" % (
         return
     elif variable.isTempVariable():
         if variable.isSharedTechnically():
-            template = CodeTemplates.template_read_shared_unclear
+            template = template_read_shared_unclear
 
             emit(
                 template % {
@@ -431,7 +466,7 @@ free variable '%s' referenced before assignment in enclosing scope""" % (
 
             return
         else:
-            template = CodeTemplates.template_read_local
+            template = template_read_local
 
             emit(
                 template % {
@@ -470,7 +505,7 @@ def getVariableDelCode(variable, tolerant, needs_check, emit, context):
         res_name = context.getIntResName()
 
         emit(
-            CodeTemplates.template_del_global_unclear % {
+            template_del_global_unclear % {
                 "module_identifier" : context.getModuleCodeName(),
                 "res_name"          : res_name,
                 "var_name"          : getConstantCode(
@@ -497,9 +532,9 @@ def getVariableDelCode(variable, tolerant, needs_check, emit, context):
     elif variable.isLocalVariable():
         if not needs_check:
             if variable.isSharedTechnically():
-                template = CodeTemplates.template_del_shared_known
+                template = template_del_shared_known
             else:
-                template = CodeTemplates.template_del_local_known
+                template = template_del_local_known
 
             emit(
                 template % {
@@ -511,9 +546,9 @@ def getVariableDelCode(variable, tolerant, needs_check, emit, context):
             )
         elif tolerant:
             if variable.isSharedTechnically():
-                template = CodeTemplates.template_del_shared_tolerant
+                template = template_del_shared_tolerant
             else:
-                template = CodeTemplates.template_del_local_tolerant
+                template = template_del_local_tolerant
 
             emit(
                 template % {
@@ -527,9 +562,9 @@ def getVariableDelCode(variable, tolerant, needs_check, emit, context):
             res_name = context.getBoolResName()
 
             if variable.isSharedTechnically():
-                template = CodeTemplates.template_del_shared_intolerant
+                template = template_del_shared_intolerant
             else:
-                template = CodeTemplates.template_del_local_intolerant
+                template = template_del_local_intolerant
 
             emit(
                 template % {
@@ -570,9 +605,9 @@ free variable '%s' referenced before assignment in enclosing scope""" % (
             # Temp variables use similar classes, can use same templates.
 
             if variable.isSharedTechnically():
-                template = CodeTemplates.template_del_shared_tolerant
+                template = template_del_shared_tolerant
             else:
-                template = CodeTemplates.template_del_local_tolerant
+                template = template_del_local_tolerant
 
             emit(
                 template % {
@@ -586,9 +621,9 @@ free variable '%s' referenced before assignment in enclosing scope""" % (
             res_name = context.getBoolResName()
 
             if variable.isSharedTechnically():
-                template = CodeTemplates.template_del_shared_intolerant
+                template = template_del_shared_intolerant
             else:
-                template = CodeTemplates.template_del_local_intolerant
+                template = template_del_local_intolerant
 
             emit(
                 template % {
@@ -600,8 +635,9 @@ free variable '%s' referenced before assignment in enclosing scope""" % (
                 }
             )
 
-            emit(
-                "assert( %s != false );" % res_name
+            getAssertionCode(
+                check = "%s != false" % res_name,
+                emit  = emit
             )
     else:
         assert False, variable
@@ -616,9 +652,9 @@ def getVariableReleaseCode(variable, needs_check, emit, context):
     # variable to NULL then, using a different template.
 
     if needs_check:
-        template = CodeTemplates.template_release_unclear
+        template = template_release_unclear
     else:
-        template = CodeTemplates.template_release_clear
+        template = template_release_clear
 
     emit(
         template % {
@@ -638,9 +674,9 @@ def getVariableInitializedCheckCode(variable, context):
 
     if variable.isLocalVariable() or variable.isTempVariable():
         if variable.isSharedTechnically():
-            template = CodeTemplates.template_check_shared
+            template = template_check_shared
         else:
-            template = CodeTemplates.template_check_local
+            template = template_check_local
 
         return template % {
             "identifier" : getVariableCode(
