@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -54,3 +54,6 @@ class ExpressionBuiltinId(ExpressionBuiltinSingleArgBase):
         # Note: Quite impossible to predict the pointer value or anything, but
         # we know the result will be a long.
         return self, None, None
+
+    def getIntValue(self):
+        return self
