@@ -22,9 +22,14 @@ to eliminate or limit their impact as much as possible, but it's difficult
 to do.
 """
 
-from nuitka import Utils
+from nuitka.Builtins import calledWithBuiltinArgumentNamesDecorator
+from nuitka.utils import Utils
 
 from .NodeBases import ExpressionChildrenHavingBase, StatementChildrenHavingBase
+from .NodeMakingHelpers import (
+    convertNoneConstantToNone,
+    makeStatementOnlyNodesFromExpressions
+)
 
 
 class ExpressionBuiltinEval(ExpressionChildrenHavingBase):
@@ -153,8 +158,6 @@ class StatementExec(StatementChildrenHavingBase):
 
     def setChild(self, name, value):
         if name in ("globals", "locals"):
-            from .NodeMakingHelpers import convertNoneConstantToNone
-
             value = convertNoneConstantToNone(value)
 
         return StatementChildrenHavingBase.setChild(self, name, value)
@@ -172,6 +175,11 @@ class StatementExec(StatementChildrenHavingBase):
         )
         source_code = self.getSourceCode()
 
+        if source_code.mayRaiseException(BaseException):
+            constraint_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
         if source_code.willRaiseException(BaseException):
             result = source_code
 
@@ -188,10 +196,13 @@ Exec statement raises implicitly when determining source code argument."""
         )
         globals_arg = self.getGlobals()
 
+        if globals_arg is not None and globals_arg.mayRaiseException(BaseException):
+            constraint_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
         if globals_arg is not None and \
            globals_arg.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     source_code,
@@ -212,10 +223,13 @@ Exec statement raises implicitly when determining globals argument."""
         )
         locals_arg = self.getLocals()
 
+        if locals_arg is not None and locals_arg.mayRaiseException(BaseException):
+            constraint_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
         if locals_arg is not None and \
            locals_arg.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     source_code,
@@ -231,10 +245,15 @@ Exec statement raises implicitly when determining globals argument."""
 Exec statement raises implicitly when determining locals argument."""
             )
 
+        constraint_collection.onExceptionRaiseExit(
+            BaseException
+        )
+
         str_value = self.getSourceCode().getStrValue()
 
         if False and str_value is not None:
-            # TODO: Don't forget to consider side effects of source code.
+            # TODO: Don't forget to consider side effects of source code,
+            # locals_arg, and globals_arg.
 
             # TODO: This needs to be re-done.
             exec_body = None
@@ -255,6 +274,7 @@ class StatementLocalsDictSync(StatementChildrenHavingBase):
         "locals",
     )
 
+    @calledWithBuiltinArgumentNamesDecorator
     def __init__(self, locals_arg, source_ref):
         StatementChildrenHavingBase.__init__(
             self,
@@ -313,5 +333,7 @@ class ExpressionBuiltinCompile(ExpressionChildrenHavingBase):
     getOptimize = ExpressionChildrenHavingBase.childGetter("optimize")
 
     def computeExpression(self, constraint_collection):
+        constraint_collection.onExceptionRaiseExit(BaseException)
+
         # TODO: Attempt for constant values to do it.
         return self, None, None
