@@ -58,8 +58,8 @@ from nuitka.__past__ import long, unicode  # pylint: disable=W0622
 from nuitka.importing import Importing
 from nuitka.importing.ImportCache import addImportedModule
 from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementReleaseVariable
+    ExpressionTargetVariableRef,
+    StatementAssignmentVariable
 )
 from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
 from nuitka.nodes.ConditionalNodes import (
@@ -85,21 +85,12 @@ from nuitka.nodes.OperatorNodes import (
     ExpressionOperationUnary
 )
 from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.StatementNodes import (
-    StatementExpressionOnly,
-    StatementsSequence
-)
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTargetVariableRef,
-    ExpressionTempVariableRef,
-    ExpressionVariableRef
-)
+from nuitka.nodes.StatementNodes import StatementExpressionOnly
+from nuitka.nodes.VariableRefNodes import ExpressionVariableRef
 from nuitka.tree import SyntaxErrors
 from nuitka.utils import Utils
 
 from .Helpers import (
-    applyLaterWrappers,
     buildNode,
     buildNodeList,
     buildStatementsNode,
@@ -109,9 +100,9 @@ from .Helpers import (
     makeDictCreationOrConstant,
     makeModuleFrame,
     makeSequenceCreationOrConstant,
+    makeStatementsSequence,
     makeStatementsSequenceFromStatement,
     makeStatementsSequenceOrStatement,
-    makeTryFinallyStatement,
     mangleName,
     mergeStatements,
     setBuildingDispatchers
@@ -133,7 +124,10 @@ from .ReformulationContractionExpressions import (
     buildSetContractionNode
 )
 from .ReformulationExecStatements import buildExecNode
-from .ReformulationFunctionStatements import buildFunctionNode
+from .ReformulationFunctionStatements import (
+    buildAsyncFunctionNode,
+    buildFunctionNode
+)
 from .ReformulationImportStatements import (
     buildImportFromNode,
     checkFutureImportsOnlyAtStart
@@ -147,10 +141,7 @@ from .ReformulationNamespacePackages import (
 from .ReformulationPrintStatements import buildPrintNode
 from .ReformulationSubscriptExpressions import buildSubscriptNode
 from .ReformulationTryExceptStatements import buildTryExceptionNode
-from .ReformulationTryFinallyStatements import (
-    buildTryFinallyNode,
-    makeTryFinallyIndicatorStatements
-)
+from .ReformulationTryFinallyStatements import buildTryFinallyNode
 from .ReformulationWithStatements import buildWithNode
 from .ReformulationYieldExpressions import buildYieldFromNode, buildYieldNode
 from .SourceReading import readSourceCodeFromFilename
@@ -193,6 +184,7 @@ def buildDictionaryNode(provider, node, source_ref):
         source_ref = source_ref
     )
 
+
 def buildConditionNode(provider, node, source_ref):
     # Conditional statements may have one or two branches. We will never see an
     # "elif", because that's already dealt with by module "ast", which turns it
@@ -247,7 +239,7 @@ def buildTryNode(provider, node, source_ref):
 
     return buildTryFinallyNode(
         provider    = provider,
-        build_tried = lambda : StatementsSequence(
+        build_tried = lambda : makeStatementsSequence(
             statements = mergeStatements(
                 (
                     buildTryExceptionNode(
@@ -255,8 +247,10 @@ def buildTryNode(provider, node, source_ref):
                         node       = node,
                         source_ref = source_ref
                     ),
-                )
+                ),
+                allow_none = True
             ),
+            allow_none = True,
             source_ref = source_ref
         ),
         node        = node,
@@ -519,6 +513,8 @@ def buildEllipsisNode(source_ref):
 
 
 def buildStatementContinueLoop(node, source_ref):
+    # Python forbids this, although technically it's probably not much of
+    # an issue.
     if getBuildContext() == "finally":
         if not Options.isFullCompat() or Utils.python_version >= 300:
             col_offset = node.col_offset - 9
@@ -537,20 +533,7 @@ def buildStatementContinueLoop(node, source_ref):
             source_line = source_line
         )
 
-
-    statements = makeTryFinallyIndicatorStatements(
-        is_loop_exit = True,
-        source_ref   = source_ref
-    )
-
-    statements.append(
-        StatementContinueLoop(
-            source_ref = source_ref
-        )
-    )
-
-    return makeStatementsSequenceOrStatement(
-        statements = statements,
+    return StatementContinueLoop(
         source_ref = source_ref
     )
 
@@ -559,19 +542,7 @@ def buildStatementBreakLoop(provider, node, source_ref):
     # A bit unusual, we need the provider, but not the node,
     # pylint: disable=W0613
 
-    statements = makeTryFinallyIndicatorStatements(
-        is_loop_exit = True,
-        source_ref   = source_ref
-    )
-
-    statements.append(
-        StatementBreakLoop(
-            source_ref = source_ref
-        )
-    )
-
-    return makeStatementsSequenceOrStatement(
-        statements = statements,
+    return StatementBreakLoop(
         source_ref = source_ref
     )
 
@@ -606,64 +577,24 @@ def buildReturnNode(provider, node, source_ref):
             user_provided = True
         )
 
-    # Indicate exceptions to potentially try/finally structures.
-    indicator_statements = makeTryFinallyIndicatorStatements(
-        is_loop_exit = False,
-        source_ref   = source_ref
+    return StatementReturn(
+        expression = expression,
+        source_ref = source_ref
     )
 
-    if indicator_statements and expression.mayRaiseException(BaseException):
-        tmp_variable = provider.allocateTempVariable(
-            temp_scope = provider.allocateTempScope("return"),
-            name       = "value"
-        )
-
-        statements = [
-                StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_variable,
-                    source_ref = expression.getSourceReference()
-                ),
-                source       = expression,
-                source_ref   = source_ref
-            )
-        ] + indicator_statements + [
-            StatementReturn(
-                expression = ExpressionTempVariableRef(
-                    variable   = tmp_variable,
-                    source_ref = expression.getSourceReference()
-                ),
-                source_ref = source_ref
-            )
-        ]
-
-        return makeTryFinallyStatement(
-            tried      = statements,
-            final      = StatementReleaseVariable(
-                variable   = tmp_variable,
-                tolerant   = True,
-                source_ref = source_ref
-            ),
-            source_ref = source_ref
-        )
-    else:
-        return makeStatementsSequenceOrStatement(
-            statements = indicator_statements + [
-                StatementReturn(
-                    expression = expression,
-                    source_ref = source_ref
-                )
-            ],
-            source_ref = source_ref
-        )
-
 
 def buildExprOnlyNode(provider, node, source_ref):
-    return StatementExpressionOnly(
+    result = StatementExpressionOnly(
         expression = buildNode(provider, node.value, source_ref),
         source_ref = source_ref
     )
 
+    result.setCompatibleSourceReference(
+        result.getExpression().getCompatibleSourceReference()
+    )
+
+    return result
+
 
 def buildUnaryOpNode(provider, node, source_ref):
     if getKind(node.op) == "Not":
@@ -714,58 +645,59 @@ def buildReprNode(provider, node, source_ref):
 def buildConditionalExpressionNode(provider, node, source_ref):
     return ExpressionConditional(
         condition      = buildNode(provider, node.test, source_ref),
-        yes_expression = buildNode(provider, node.body, source_ref),
-        no_expression  = buildNode(provider, node.orelse, source_ref),
+        expression_yes = buildNode(provider, node.body, source_ref),
+        expression_no  = buildNode(provider, node.orelse, source_ref),
         source_ref     = source_ref
     )
 
 
 setBuildingDispatchers(
     path_args3 = {
-        "Name"         : buildVariableReferenceNode,
-        "Assign"       : buildAssignNode,
-        "Delete"       : buildDeleteNode,
-        "Lambda"       : buildLambdaNode,
-        "GeneratorExp" : buildGeneratorExpressionNode,
-        "If"           : buildConditionNode,
-        "While"        : buildWhileLoopNode,
-        "For"          : buildForLoopNode,
-        "Compare"      : buildComparisonNode,
-        "ListComp"     : buildListContractionNode,
-        "DictComp"     : buildDictContractionNode,
-        "SetComp"      : buildSetContractionNode,
-        "Dict"         : buildDictionaryNode,
-        "Set"          : buildSequenceCreationNode,
-        "Tuple"        : buildSequenceCreationNode,
-        "List"         : buildSequenceCreationNode,
-        "Global"       : handleGlobalDeclarationNode,
-        "Nonlocal"     : handleNonlocalDeclarationNode,
-        "TryExcept"    : buildTryExceptionNode,
-        "TryFinally"   : _buildTryFinallyNode,
-        "Try"          : buildTryNode,
-        "Raise"        : buildRaiseNode,
-        "Import"       : buildImportModulesNode,
-        "ImportFrom"   : buildImportFromNode,
-        "Assert"       : buildAssertNode,
-        "Exec"         : buildExecNode,
-        "With"         : buildWithNode,
-        "FunctionDef"  : buildFunctionNode,
-        "ClassDef"     : buildClassNode,
-        "Print"        : buildPrintNode,
-        "Call"         : buildCallNode,
-        "Subscript"    : buildSubscriptNode,
-        "BoolOp"       : buildBoolOpNode,
-        "Attribute"    : buildAttributeNode,
-        "Return"       : buildReturnNode,
-        "Yield"        : buildYieldNode,
-        "YieldFrom"    : buildYieldFromNode,
-        "Expr"         : buildExprOnlyNode,
-        "UnaryOp"      : buildUnaryOpNode,
-        "BinOp"        : buildBinaryOpNode,
-        "Repr"         : buildReprNode,
-        "AugAssign"    : buildInplaceAssignNode,
-        "IfExp"        : buildConditionalExpressionNode,
-        "Break"        : buildStatementBreakLoop,
+        "Name"              : buildVariableReferenceNode,
+        "Assign"            : buildAssignNode,
+        "Delete"            : buildDeleteNode,
+        "Lambda"            : buildLambdaNode,
+        "GeneratorExp"      : buildGeneratorExpressionNode,
+        "If"                : buildConditionNode,
+        "While"             : buildWhileLoopNode,
+        "For"               : buildForLoopNode,
+        "Compare"           : buildComparisonNode,
+        "ListComp"          : buildListContractionNode,
+        "DictComp"          : buildDictContractionNode,
+        "SetComp"           : buildSetContractionNode,
+        "Dict"              : buildDictionaryNode,
+        "Set"               : buildSequenceCreationNode,
+        "Tuple"             : buildSequenceCreationNode,
+        "List"              : buildSequenceCreationNode,
+        "Global"            : handleGlobalDeclarationNode,
+        "Nonlocal"          : handleNonlocalDeclarationNode,
+        "TryExcept"         : buildTryExceptionNode,
+        "TryFinally"        : _buildTryFinallyNode,
+        "Try"               : buildTryNode,
+        "Raise"             : buildRaiseNode,
+        "Import"            : buildImportModulesNode,
+        "ImportFrom"        : buildImportFromNode,
+        "Assert"            : buildAssertNode,
+        "Exec"              : buildExecNode,
+        "With"              : buildWithNode,
+        "FunctionDef"       : buildFunctionNode,
+        "AsyncFunctionDef"  : buildAsyncFunctionNode,
+        "ClassDef"          : buildClassNode,
+        "Print"             : buildPrintNode,
+        "Call"              : buildCallNode,
+        "Subscript"         : buildSubscriptNode,
+        "BoolOp"            : buildBoolOpNode,
+        "Attribute"         : buildAttributeNode,
+        "Return"            : buildReturnNode,
+        "Yield"             : buildYieldNode,
+        "YieldFrom"         : buildYieldFromNode,
+        "Expr"              : buildExprOnlyNode,
+        "UnaryOp"           : buildUnaryOpNode,
+        "BinOp"             : buildBinaryOpNode,
+        "Repr"              : buildReprNode,
+        "AugAssign"         : buildInplaceAssignNode,
+        "IfExp"             : buildConditionalExpressionNode,
+        "Break"             : buildStatementBreakLoop,
     },
     path_args2 = {
         "NameConstant" : buildNamedConstantNode,
@@ -971,15 +903,11 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
 
 
     if is_module:
-        result = makeModuleFrame(
+        return makeModuleFrame(
             module     = provider,
             statements = statements,
             source_ref = source_ref
         )
-
-        applyLaterWrappers()
-
-        return result
     else:
         assert False
 
@@ -1091,17 +1019,38 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
     return result, source_ref, source_filename
 
 
+class CodeTooComplexCode(Exception):
+    """ The code of the module is too complex.
+
+        It cannot be compiled, with recursive code, and therefore the bytecode
+        should be used instead.
+
+        Example of this is "idnadata".
+    """
+
+    pass
+
+
 def createModuleTree(module, source_ref, source_code, is_main):
     if Options.isShowProgress():
         memory_watch = Utils.MemoryWatch()
 
-    module_body = buildParseTree(
-        provider    = module,
-        source_code = source_code,
-        source_ref  = source_ref,
-        is_module   = True,
-        is_main     = is_main
-    )
+    try:
+        module_body = buildParseTree(
+            provider    = module,
+            source_code = source_code,
+            source_ref  = source_ref,
+            is_module   = True,
+            is_main     = is_main
+        )
+    except RuntimeError as e:
+        if "maximum recursion depth" in e.args[0]:
+            raise CodeTooComplexCode(
+                module.getFullName(),
+                module.getCompileTimeFilename()
+            )
+
+        raise
 
     if module_body.isStatementsFrame():
         module_body = makeStatementsSequenceFromStatement(
@@ -1144,7 +1093,7 @@ def buildModuleTree(filename, package, is_top, is_main):
         createModuleTree(
             module      = module,
             source_ref  = source_ref,
-            source_code = readSourceCodeFromFilename(module, source_filename),
+            source_code = readSourceCodeFromFilename(module.getFullName(), source_filename),
             is_main     = is_main
         )
 
