@@ -22,31 +22,54 @@ source code comments with developer manual sections.
 
 """
 
-from nuitka.nodes.CallNodes import ExpressionCall
+from nuitka.nodes.CallNodes import makeExpressionCall
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
 from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionCall,
     ExpressionFunctionCreation,
     ExpressionFunctionRef
 )
+from nuitka.PythonVersions import python_version
 
 from .Helpers import (
     buildNode,
-    buildNodeList,
+    getKind,
     makeDictCreationOrConstant,
     makeSequenceCreationOrConstant
 )
 
 
 def buildCallNode(provider, node, source_ref):
-    positional_args = buildNodeList(provider, node.args, source_ref)
+    if python_version >= 350:
+        list_star_arg = None
+        dict_star_arg = None
+
+    positional_args = []
+
+    for node_arg in node.args:
+        if getKind(node_arg) == "Starred":
+            assert python_version >= 350
+
+            list_star_arg = buildNode(provider, node_arg.value, source_ref)
+            continue
+
+        positional_args.append(
+            buildNode(provider, node_arg, source_ref)
+        )
 
     # Only the values of keyword pairs have a real source ref, and those only
     # really matter, so that makes sense.
     keys = []
     values = []
 
+
     for keyword in node.keywords:
+        if keyword.arg is None:
+            assert python_version >= 350
+
+            dict_star_arg = buildNode(provider, keyword.value, source_ref)
+            continue
+
         keys.append(
             ExpressionConstantRef(
                 constant      = keyword.arg,
@@ -58,8 +81,9 @@ def buildCallNode(provider, node, source_ref):
             buildNode(provider, keyword.value, source_ref)
         )
 
-    list_star_arg = buildNode(provider, node.starargs, source_ref, True)
-    dict_star_arg = buildNode(provider, node.kwargs, source_ref, True)
+    if python_version < 350:
+        list_star_arg = buildNode(provider, node.starargs, source_ref, True)
+        dict_star_arg = buildNode(provider, node.kwargs, source_ref, True)
 
     return _makeCallNode(
         called          = buildNode(provider, node.func, source_ref),
@@ -78,7 +102,7 @@ def _makeCallNode(called, positional_args, keys, values, list_star_arg,
     # pylint: disable=R0914
 
     if list_star_arg is None and dict_star_arg is None:
-        result = ExpressionCall(
+        result = makeExpressionCall(
             called     = called,
             args       = makeSequenceCreationOrConstant(
                 sequence_kind = "tuple",
@@ -170,6 +194,9 @@ def _makeCallNode(called, positional_args, keys, values, list_star_arg,
                 )
             )
 
+        if python_version >= 350 and list_star_arg is not None:
+            helper_args.append(list_star_arg)
+
         if keys:
             helper_args.append(
                 makeDictCreationOrConstant(
@@ -180,7 +207,7 @@ def _makeCallNode(called, positional_args, keys, values, list_star_arg,
                 )
             )
 
-        if list_star_arg is not None:
+        if python_version < 350 and list_star_arg is not None:
             helper_args.append(list_star_arg)
 
         if dict_star_arg is not None:
