@@ -23,6 +23,8 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
+    ExpressionTargetTempVariableRef,
+    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
@@ -57,10 +59,6 @@ from nuitka.nodes.StatementNodes import (
     StatementGeneratorEntry,
     StatementsSequence
 )
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef
-)
 from nuitka.nodes.YieldNodes import ExpressionYield
 from nuitka.utils import Utils
 
