@@ -23,6 +23,8 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
+    ExpressionTargetTempVariableRef,
+    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
@@ -34,25 +36,22 @@ from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
 from nuitka.nodes.ConditionalNodes import StatementConditional
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
 from nuitka.nodes.LoopNodes import StatementBreakLoop, StatementLoop
+from nuitka.nodes.OperatorNodes import ExpressionOperationNOT
 from nuitka.nodes.StatementNodes import StatementsSequence
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef
-)
 
 from .Helpers import (
     buildNode,
     buildStatementsNode,
     makeStatementsSequence,
     makeStatementsSequenceFromStatement,
-    makeTryFinallyStatement,
+    makeStatementsSequenceFromStatements,
+    mergeStatements,
     popBuildContext,
-    popIndicatorVariable,
-    pushBuildContext,
-    pushIndicatorVariable
+    pushBuildContext
 )
 from .ReformulationAssignmentStatements import buildAssignmentStatements
 from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
 def buildForLoopNode(provider, node, source_ref):
@@ -117,6 +116,7 @@ def buildForLoopNode(provider, node, source_ref):
 
     statements = (
         makeTryExceptSingleHandlerNode(
+            provider       = handler_body,
             tried          = makeStatementsSequenceFromStatement(
                 statement = StatementAssignmentVariable(
                     variable_ref = ExpressionTargetTempVariableRef(
@@ -135,7 +135,6 @@ def buildForLoopNode(provider, node, source_ref):
             ),
             exception_name = "StopIteration",
             handler_body   = handler_body,
-            public_exc     = False,
             source_ref     = source_ref
         ),
         buildAssignmentStatements(
@@ -150,7 +149,6 @@ def buildForLoopNode(provider, node, source_ref):
     )
 
     pushBuildContext("loop_body")
-    pushIndicatorVariable(None)
     statements += (
         buildStatementsNode(
             provider   = provider,
@@ -158,7 +156,6 @@ def buildForLoopNode(provider, node, source_ref):
             source_ref = source_ref
         ),
     )
-    popIndicatorVariable()
     popBuildContext()
 
     loop_body = makeStatementsSequence(
@@ -170,12 +167,10 @@ def buildForLoopNode(provider, node, source_ref):
     cleanup_statements = [
         StatementReleaseVariable(
             variable   = tmp_value_variable,
-            tolerant   = True,
             source_ref = source_ref
         ),
         StatementReleaseVariable(
             variable   = tmp_iter_variable,
-            tolerant   = True,
             source_ref = source_ref
         )
     ]
@@ -211,6 +206,7 @@ def buildForLoopNode(provider, node, source_ref):
             source_ref   = source_ref
         ),
         makeTryFinallyStatement(
+            provider   = provider,
             tried      = StatementLoop(
                 body       = loop_body,
                 source_ref = source_ref
@@ -245,19 +241,18 @@ def buildForLoopNode(provider, node, source_ref):
 
         statements = (
             makeTryFinallyStatement(
+                provider   = provider,
                 tried      = statements,
                 final      = StatementReleaseVariable(
                     variable   = tmp_break_indicator,
-                    tolerant   = False,
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
             ),
         )
 
-    return StatementsSequence(
-        statements = statements,
-        source_ref = source_ref
+    return makeStatementsSequenceFromStatements(
+        *statements
     )
 
 
@@ -305,13 +300,11 @@ def buildWhileLoopNode(provider, node, source_ref):
         )
 
     pushBuildContext("loop_body")
-    pushIndicatorVariable(None)
     loop_statements = buildStatementsNode(
         provider   = provider,
         nodes      = node.body,
         source_ref = source_ref
     )
-    popIndicatorVariable()
     popBuildContext()
 
     # The loop body contains a conditional statement at the start that breaks
@@ -319,12 +312,15 @@ def buildWhileLoopNode(provider, node, source_ref):
     loop_body = makeStatementsSequence(
         statements = (
             StatementConditional(
-                condition  = buildNode(provider, node.test, source_ref),
-                no_branch  = StatementsSequence(
+                condition  = ExpressionOperationNOT(
+                    operand    = buildNode(provider, node.test, source_ref),
+                    source_ref = source_ref,
+                ),
+                yes_branch = StatementsSequence(
                     statements = statements,
                     source_ref = source_ref
                 ),
-                yes_branch = None,
+                no_branch  = None,
                 source_ref = source_ref
             ),
             loop_statements
@@ -374,10 +370,10 @@ def buildWhileLoopNode(provider, node, source_ref):
 
         statements = (
             makeTryFinallyStatement(
+                provider   = provider,
                 tried      = statements,
                 final      = StatementReleaseVariable(
                     variable   = tmp_break_indicator,
-                    tolerant   = False,
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
@@ -385,6 +381,6 @@ def buildWhileLoopNode(provider, node, source_ref):
         )
 
         return StatementsSequence(
-            statements = statements,
+            statements = mergeStatements(statements, False),
             source_ref = source_ref
         )
