@@ -21,7 +21,10 @@ Namespace packages of Python3.3
 """
 
 from nuitka import Options
-from nuitka.nodes.AssignNodes import StatementAssignmentVariable
+from nuitka.nodes.AssignNodes import (
+    ExpressionTargetVariableRef,
+    StatementAssignmentVariable
+)
 from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
 from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
@@ -39,7 +42,6 @@ from nuitka.nodes.ModuleNodes import (
     ExpressionModuleFileAttributeRef,
     PythonPackage
 )
-from nuitka.nodes.VariableRefNodes import ExpressionTargetVariableRef
 from nuitka.SourceCodeReferences import SourceCodeReference
 from nuitka.utils import Utils
 
