@@ -24,12 +24,14 @@ source code comments with developer manual sections.
 
 from nuitka import Options
 from nuitka.nodes.AssignNodes import (
+    ExpressionTargetTempVariableRef,
+    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
 from nuitka.nodes.AttributeNodes import (
     ExpressionAttributeLookup,
-    ExpressionSpecialAttributeLookup
+    ExpressionAttributeLookupSpecial
 )
 from nuitka.nodes.CallNodes import ExpressionCallEmpty, ExpressionCallNoKeywords
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
@@ -41,31 +43,35 @@ from nuitka.nodes.ExceptionNodes import (
     ExpressionCaughtExceptionTypeRef,
     ExpressionCaughtExceptionValueRef
 )
-from nuitka.nodes.NodeMakingHelpers import makeReraiseExceptionStatement
 from nuitka.nodes.StatementNodes import (
     StatementExpressionOnly,
     StatementsSequence
 )
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef
-)
+from nuitka.tree.Helpers import makeReraiseExceptionStatement
 from nuitka.utils import Utils
 
 from .Helpers import (
     buildNode,
     buildStatementsNode,
+    getKind,
+    makeConditionalStatement,
     makeStatementsSequence,
-    makeStatementsSequenceFromStatement,
-    makeTryFinallyStatement
+    makeStatementsSequenceFromStatement
 )
 from .ReformulationAssignmentStatements import buildAssignmentStatements
 from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
-def _buildWithNode(provider, context_expr, assign_target, body, source_ref):
+def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
+                   source_ref):
+    # Many details, pylint: disable=R0914
+
     with_source = buildNode(provider, context_expr, source_ref)
 
+    if Options.isFullCompat():
+        source_ref = with_source.getCompatibleSourceReference()
+
     temp_scope = provider.allocateTempScope("with")
 
     tmp_source_variable = provider.allocateTempVariable(
@@ -105,19 +111,25 @@ def _buildWithNode(provider, context_expr, assign_target, body, source_ref):
         source_ref = source_ref
     )
 
-    if Options.isFullCompat() and with_body is not None:
-        with_exit_source_ref = with_body.getStatements()[-1].\
-          getSourceReference()
+    if Options.isFullCompat():
+        if body:
+            deepest = body
+
+            while deepest.getVisitableNodes():
+                deepest = deepest.getVisitableNodes()[-1]
+
+            body_lineno = deepest.getCompatibleSourceReference().getLineNumber()
+
+        with_exit_source_ref = source_ref.atLineNumber(body_lineno)
     else:
         with_exit_source_ref = source_ref
 
-
     # The "__enter__" and "__exit__" were normal attribute lookups under
     # CPython2.6, but that changed with CPython2.7.
     if Utils.python_version < 270:
         attribute_lookup_class = ExpressionAttributeLookup
     else:
-        attribute_lookup_class = ExpressionSpecialAttributeLookup
+        attribute_lookup_class = ExpressionAttributeLookupSpecial
 
     statements = [
         # First assign the with context to a temporary variable.
@@ -177,11 +189,11 @@ def _buildWithNode(provider, context_expr, assign_target, body, source_ref):
         ),
     ]
 
-    source_ref = source_ref.atInternal()
-
     statements += [
         makeTryFinallyStatement(
+            provider   = provider,
             tried      = makeTryExceptSingleHandlerNode(
+                provider       = provider,
                 tried          = with_body,
                 exception_name = "BaseException",
                 handler_body   = StatementsSequence(
@@ -199,19 +211,19 @@ def _buildWithNode(provider, context_expr, assign_target, body, source_ref):
                             ),
                             source_ref   = source_ref
                         ),
-                        StatementConditional(
+                        makeConditionalStatement(
                             condition  = ExpressionCallNoKeywords(
                                 called     = ExpressionTempVariableRef(
                                     variable   = tmp_exit_variable,
-                                    source_ref = source_ref
+                                    source_ref = with_exit_source_ref
                                 ),
                                 args       = ExpressionMakeTuple(
                                     elements   = (
                                         ExpressionCaughtExceptionTypeRef(
-                                            source_ref = source_ref
+                                            source_ref = with_exit_source_ref
                                         ),
                                         ExpressionCaughtExceptionValueRef(
-                                            source_ref = source_ref
+                                            source_ref = with_exit_source_ref
                                         ),
                                         ExpressionCaughtExceptionTracebackRef(
                                             source_ref = source_ref
@@ -219,15 +231,13 @@ def _buildWithNode(provider, context_expr, assign_target, body, source_ref):
                                     ),
                                     source_ref = source_ref
                                 ),
-                                source_ref = source_ref
+                                source_ref = with_exit_source_ref
                             ),
-                            no_branch  = makeStatementsSequenceFromStatement(
-                                statement = makeReraiseExceptionStatement(
-                                    source_ref = source_ref
-                                )
+                            no_branch  = makeReraiseExceptionStatement(
+                                source_ref = with_exit_source_ref
                             ),
                             yes_branch = None,
-                            source_ref = source_ref
+                            source_ref = with_exit_source_ref
                         ),
                     ),
                     source_ref = source_ref
@@ -271,27 +281,24 @@ def _buildWithNode(provider, context_expr, assign_target, body, source_ref):
     ]
 
     return makeTryFinallyStatement(
+        provider   = provider,
         tried      = statements,
         final      = (
             StatementReleaseVariable(
                 variable   = tmp_source_variable,
-                tolerant   = True,
-                source_ref = source_ref
+                source_ref = with_exit_source_ref
             ),
             StatementReleaseVariable(
                 variable   = tmp_enter_variable,
-                tolerant   = True,
-                source_ref = source_ref
+                source_ref = with_exit_source_ref
             ),
             StatementReleaseVariable(
                 variable   = tmp_exit_variable,
-                tolerant   = True,
-                source_ref = source_ref
+                source_ref = with_exit_source_ref
             ),
             StatementReleaseVariable(
                 variable   = tmp_indicator_variable,
-                tolerant   = True,
-                source_ref = source_ref
+                source_ref = with_exit_source_ref
             ),
         ),
         source_ref = source_ref
@@ -322,10 +329,18 @@ def buildWithNode(provider, node, source_ref):
     context_exprs.reverse()
     assign_targets.reverse()
 
+    # For compatibility, we need to gather a line number for the body here
+    # already, but only the full compatibility mode will use it.
+    terminal_statement = node.body[-1]
+    while getKind(terminal_statement) == "With":
+        terminal_statement = terminal_statement.body[-1]
+    body_lineno = terminal_statement.lineno
+
     for context_expr, assign_target in zip(context_exprs, assign_targets):
         body = _buildWithNode(
             provider      = provider,
             body          = body,
+            body_lineno   = body_lineno,
             context_expr  = context_expr,
             assign_target = assign_target,
             source_ref    = source_ref
