@@ -18,7 +18,7 @@
 
 from __future__ import print_function
 
-import inspect, types, sys
+import inspect, types, sys, pprint
 
 def compiledFunction(a, b):
     pass
@@ -83,7 +83,6 @@ def someFunction(a):
 someFunction(2)
 
 class C:
-    print("Class frame", sys._getframe().f_code)
     print("Class locals", str(sys._getframe().f_locals).replace(", '__locals__': {...}", "").replace("'__qualname__': 'C', ", ""))
     print("Class flags", sys._getframe().f_code.co_flags)
 
@@ -96,9 +95,13 @@ f()
 def displayDict(d):
     if "__loader__" in d:
         d = dict(d)
-        d["__loader__"] = "<loader removed>"
+        d["__loader__"] = "<__loader__ removed>"
 
-    return repr(d)
+    if "__file__" in d:
+        d = dict(d)
+        d["__file__"] = "<__file__ removed>"
+
+    return pprint.pformat(d)
 
 print("Module frame locals", displayDict(sys._getframe().f_locals))
 print("Module flags", sys._getframe().f_code.co_flags)
