@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Softwar where
@@ -17,9 +17,11 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+import warnings
+warnings.filterwarnings("ignore", "")
+
 import sys
 import gtk
 
 import pygtk
-pygtk.require('2.0')
-
+pygtk.require("2.0")
