@@ -42,7 +42,7 @@ from test_common import (
     createSearchMode
 )
 
-python_version = setup( needs_io_encoding = True)
+python_version = setup(needs_io_encoding = True)
 
 search_mode = createSearchMode()
 
@@ -52,7 +52,7 @@ search_mode.mayFailFor(
     "PySideUsing.py"
 )
 
-for filename in sorted(os.listdir(".")):
+for filename in sorted(os.listdir('.')):
     if not filename.endswith(".py"):
         continue
 
@@ -89,7 +89,7 @@ for filename in sorted(os.listdir(".")):
             continue
 
         # For the warnings.
-        extra_flags.append( "ignore_stderr" )
+        extra_flags.append("ignore_stderr")
 
     if "PyQt4" in filename:
         # Don't test on platforms not supported by current Debian testing, and
@@ -106,8 +106,8 @@ for filename in sorted(os.listdir(".")):
             )
             continue
 
-        # For the warnings.
-        extra_flags.append( "ignore_stderr" )
+        # For the plug-in information.
+        extra_flags.append("ignore_infos")
 
     if "PyQt5" in filename:
         # Don't test on platforms not supported by current Debian testing, and
@@ -124,13 +124,19 @@ for filename in sorted(os.listdir(".")):
             )
             continue
 
-        # For the warnings.
-        extra_flags.append( "ignore_stderr" )
+        # For the plug-in information.
+        extra_flags.append("ignore_infos")
 
     # TODO: Temporary only
     if os.name == "nt" and "PyQt" in filename:
         continue
 
+    if "PySide" in filename or "PyQt" in filename:
+        extra_flags.append("plugin_enable:qt-plugins")
+
+    if filename == "CtypesUsing.py":
+        extra_flags.append("plugin_disable:pylint-warnings")
+
     if filename == "GtkUsing.py":
         # Don't test on platforms not supported by current Debian testing, and
         # which should be considered irrelevant by now.
@@ -147,7 +153,7 @@ for filename in sorted(os.listdir(".")):
             continue
 
         # For the warnings.
-        extra_flags.append( "ignore_stderr" )
+        extra_flags.append("ignore_stderr")
 
     if filename.startswith("Win"):
         if os.name != "nt":
@@ -170,6 +176,14 @@ for filename in sorted(os.listdir(".")):
             )
             continue
 
+    if filename == "TkInterUsing.py":
+        if not hasModule("tkinter"):
+            my_print(
+                "Skipping", filename, "tkinter not installed for",
+                python_version, "but test needs it."
+            )
+            continue
+
     if filename not in ("PySideUsing.py", "PyQt4Using.py", "PyQt5Using.py",
                         "PyQt4Plugins.py", "PyQt5Plugins.py", "GtkUsing.py",
                         "LxmlUsing.py", "Win32ComUsing.py"):
@@ -373,7 +387,14 @@ for filename in sorted(os.listdir(".")):
             "LPK.DLL", "USP10.DLL", "CFGMGR32.DLL", "MSIMG32.DLL",
             "POWRPROF.DLL", "SETUPAPI.DLL", "WINSTA.DLL", "CRYPT32.DLL",
             "IPHLPAPI.DLL", "MPR.DLL", "CREDUI.DLL", "NETPLWIZ.DLL",
-            "OLE32.DLL", "ACTIVEDS.DLL", "ADSLDPC.DLL", "USERENV.DLL"):
+            "OLE32.DLL", "ACTIVEDS.DLL", "ADSLDPC.DLL", "USERENV.DLL",
+            "APPREPAPI.DLL", "BCP47LANGS.DLL", "BCRYPTPRIMITIVES.DLL",
+            "CERTCA.DLL", "CHARTV.DLL", "COMBASE.DLL", "COML2.DLL",
+            "DCOMP.DLL", "DPAPI.DLL", "DSPARSE.DLL", "FECLIENT.DLL",
+            "FIREWALLAPI.DLL", "FLTLIB.DLL", "MRMCORER.DLL", "NTASN1.DLL",
+            "SECHOST.DLL", "SETTINGSYNCPOLICY.DLL", "SHCORE.DLL", "TBS.DLL",
+            "TWINAPI.APPCORE.DLL", "TWINAPI.DLL", "VIRTDISK.DLL",
+            "WEBSOCKET.DLL", "WEVTAPI.DLL", "WINMMBASE.DLL", "WMICLNT.DLL"):
             continue
 
         # Win API can be assumed.
